# -*- coding: cp1250 -*-

"""
This script contains a "fill_DEM" function that loads the DEM raster and 
and creates a path from each bottom of the depression bottom to the outside
of the depression.

Tento skript obsahuje funkci "Carving", kter nate rastr DEM a vytvo cestu
od kadho dna dna deprese smrem ven z prohlubn. 
"""

# Load modules.
# Naten modul.
import arcpy, numpy, os, sys, rta, heap, neighbors, border_of_raster

arcpy.env.overwriteOutput = True

# Linking the parameters specified in  the window toolbox to the script.
# Propojen parametr zadanch v okn nstroje se skriptem.
raster = sys.argv[1]
output = sys.argv[2]


def Carving(raster, output):

    # Loads the raster to numpy.array.
    # Naten rastru do numpy.array
    aar_dem = rta.raster_to_numpy(raster)

    DEM = aar_dem[0]
    Xmax = aar_dem[1]
    Ymax = aar_dem[2]
    vCell = aar_dem[3]
    LeftX = aar_dem[4]
    LeftY = aar_dem[5]

    # Creates a zero array of the same size as a array created from a input raster.
    # Vytvoen pole nul o stejn velikosti, jako pole vytvoen ze vstupnho rastru
    ra = numpy.zeros((Xmax,Ymax))   # Indicates which cells are processed. (Oznauje, kter buky jsou zpracovny.)
    rx = numpy.zeros((Xmax,Ymax))
    ry = numpy.zeros((Xmax,Ymax))

    # Creates a heap (x,y,z) in the list sorted by the "z" value from values on the border of the  input raster.
    # Vytvoen haldy (x,y,z) v seznamu azenou podle hodnoty "z" z hodnot na okraji vstupnho rastru.
    ha = []
    border_of_raster.border(DEM,ha,Xmax,Ymax)

    # Checks the neighbors of the first element in the heap.
    # Kontrola soused prvnho prvku v halde.
    while len(ha) != 0:
        dep = False
        ra[ha[0][0],ha[0][1]] = 1  # Sets the first element in the heap as processed. (Nastaven prvnho prvku v halde jako zpracovan.) 
        sou = neighbors.neighbors_z(DEM,ha[0][0],ha[0][1],Xmax,Ymax)    # Detects coordinates and elevation of neighboring cells. (Zjitn souadnic a elevac sousednch bunk                

        # Checking if a cell is a potential bottom of depression.
        # Kontrola, jestli buka nen potencionln dno deprese.
        if ha[0][0] != 0:
            if ha[0][0] != Xmax-1:
                if ha[0][1] != 0:
                    if ha[0][1] != Ymax-1:
                        List = []
                        for i in sou:
                            if i[2] > ha[0][2]:
                                List.append(1)
                            if i[2] == ha[0][2]:
                                List.append(2)
                            if i[2] < ha[0][2]:
                                List.append(3)
                        if 3 not in List and 2 not in List and 1 in List:
                            dep = True
                        if 1 in List and 2 in List and 3 not in List:
                            dep = True
                        del(List)

        # Create a pathway from the bottom of depression out of depression.
        # Vytvoen cesty ode dna deprese ven z deprese.
        if dep == True:
            x = ha[0][0] 
            y = ha[0][1]
            stop = False
            while not stop:
                if DEM[x,y] <= DEM[int(rx[x,y]),int(ry[x,y])]:
                    DEM[int(rx[x,y]),int(ry[x,y])] = DEM[x,y]
                    x2 = x
                    x = int(rx[x,y])
                    y = int(ry[x2,y])
                    del (x2)
                else:
                    stop = True
                if x == 0 or x == Xmax-1 or  y == 0 or  y == Ymax-1:
                    stop = True
                if DEM[x,y] == -999:
                    stop = True
            del(x)
            del(y)

        x = ha[0][0] 
        y = ha[0][1]

        # Removes the first element in the heap and replaces the next with the smallest elevation value.
        # Odstrann prvnho prvku v hald a nahrazen dalm s nejmen hodnotou elevace.
        heap.ret_min(ha)
        
        # Unprocessed neighbors are added link to the processed cell.
        # Nezpracovanm sousedm se pid odkaz na zpracovvanou buku. 
        for i in sou:
            if ra[i[0],i[1]] != 1:  # Checks if the neighbor is not already processed. (Kontrola souseda jestli nen u zpracovan).
                if i[2] != -999:
                    ra[i[0],i[1]] = 1
                    rx[i[0],i[1]] = int(x)  
                    ry[i[0],i[1]] = int(y)
                    if i not in ha:     # If a neighbor is not in a heap, he adds it to the heap. (Pokud soused nen v hald, tak se pid do haldy.) 
                        heap.ins_el(ha, [i[0], i[1], DEM[i[0],i[1]]], index=2)
        
    # Convert numpy.array to raster and save.
    # (Peveden numpy.array na rastr a jeho uloen.)  
    Fin_DEM = arcpy.NumPyArrayToRaster(DEM,arcpy.Point(LeftX,LeftY),vCell,value_to_nodata = -999) 
    Fin_DEM.save(output)
         

pls = Carving(raster, output)











