# -*- coding: cp1250 -*-

"""
This script contains a "Dep_layer" function that loads the DEM raster and creates a polygon
layer of depressions and flat areas. In the attribute table of the newly created layer fills
in information (depression number, maximum elevation, minimum elevation, maximum depression
depth, x and y depression coordinates) about each depression.

Tento skript obsahuje funkci "Dep_layer", kter nate rastr DEM a vytvo polygonovou vrstvu
depres a ploin. V atributov tabulky nov vytvoen vrstvy vypln informace (slo deprese,
maximln elevac, minimln elevaci, maximln hloubku deprese, x a y souadnici deprese) o
kad depresi.  
"""

# Load modules.
# Naten modul.
import arcpy, numpy, sys, os, rta, heap, neighbors, border_of_raster
from arcpy.sa import *

arcpy.env.overwriteOutput = True

# Linking the parameters specified in  the window toolbox to the script.
# Propojen parametr zadanch v okn nstroje se skriptem.
raster = sys.argv[1]
output = sys.argv[2]
output_flat = sys.argv[3]


def Dep_layer(raster, output, output_flat):

    # Finding the path to the input raster.
    # Zjitn cesty k vstupnmu rastru.
    location = arcpy.Describe(raster)
    path = location.path
    
    # Loads the raster to numpy.array.
    # Naten rastru do numpy.array
    aar_dem = rta.raster_to_numpy(raster)

    DEM = aar_dem[0]
    Xmax = aar_dem[1]
    Ymax = aar_dem[2]
    vCell = aar_dem[3]
    LeftX = aar_dem[4]
    LeftY = aar_dem[5]

    # Creates a zero array of the same size as the converted raster.
    # Vytvoen pole nul o stejn velikosti, jako peveden rastr.
    ra = numpy.zeros((Xmax,Ymax)) # Indicates which cells are processed. (Oznauje, kter buky jsou zpracovny.)
    rb = numpy.zeros((Xmax,Ymax))
    rc = numpy.zeros((Xmax,Ymax))

    # Creates a heap (x,y,z) sorted by the "z" value from values on the border of the  input raster.
    # Vytvoen haldy (x,y,z) azenou podle hodnoty "z" z hodnot na okraji vstupnho rastru.
    ha = []
    border_of_raster.border(DEM,ha,Xmax,Ymax)

    # Checks the neighbors of the first element in the heap.
    # Kontrola soused prvnho prvku v halde.
    List1 = []
    List2 = []
    y = 1
    x = 1
    while len(ha) != 0:
        ra[ha[0][0],ha[0][1]] = 1  # Sets the first element in the heap as processed. (Nastaven prvnho prvku v halde jako zpracovan.)
        sou = neighbors.neighbors_z(DEM,ha[0][0],ha[0][1],Xmax,Ymax)    # Detects coordinates and elevation of neighboring cells. (Zjitn souadnic a elevac sousednch bunk.)
        for i in sou:
            if ra[i[0],i[1]] != 1:  # Checks if the neighbor is not already processed. (Kontrola souseda, jestli nen u zpracovan.)
                if i[2] != -999:   # Check if the neighbors contains NoData. (Kontrola, jestli soused neobsahuje NoData.)
                    if i[2] < ha[0][2]:             
                        ra[i[0],i[1]] = 1
                        # Getting basic data about depression.
                        # Zskn zkladnch daj o depresi.  
                        if rb[ha[0][0],ha[0][1]] == 0:
                            rb[i[0],i[1]] = x
                            list1 = [x, ha[0][2], i[2], float(ha[0][2])-float(i[2]), i[0], i[1]]
                            List1.append(list1)
                            del(list1)
                            x = x+1      
                        else:
                            rb[i[0],i[1]] = rb[ha[0][0],ha[0][1]]
                            if i[2] < List1[int(rb[i[0],i[1]])-1][2]:
                                List1[int(rb[i[0],i[1]])-1][2] = i[2]
                                List1[int(rb[i[0],i[1]])-1][3] = float(ha[0][2])-float(i[2])
                                List1[int(rb[i[0],i[1]])-1][4] = i[0]
                                List1[int(rb[i[0],i[1]])-1][5] = i[1]
                        # Override neighbor elevation to processed cell elevation value.
                        # Pepsn elevace souseda na hodnotu elevace zpracovvan buky.
                        DEM[i[0],i[1]] = ha[0][2]
                    if i[2] == ha[0][2]:             
                        ra[i[0],i[1]] = 1
                        # Getting basic data about flat areas. 
                        # Zskn zkladnch daj o ploin.  
                        if rc[ha[0][0],ha[0][1]] == 0:
                            rc[ha[0][0],ha[0][1]] = y
                            rc[i[0],i[1]] = y
                            list2 = [y, ha[0][2]]
                            List2.append(list2)
                            del(list2)
                            y = y+1     
                        else:
                            rc[i[0],i[1]] = rc[ha[0][0],ha[0][1]]
                    if i not in ha:
                        # If neighbor is not in the heap, it will be added.
                        # Pokud nen soused v hald, pid se.
                        heap.ins_el(ha, [i[0], i[1], DEM[i[0],i[1]]], index=2)
        # Removes the first element in the heap and replaces the next with the smallest elevation value.
        # Odstrann prvnho prvku v hald a nahrazen dalm s nejmen hodnotou.
        heap.ret_min(ha)

    # Turn on extension to the tools "Spatial".
    # Zapne extenzi k nstrojm "Spatial".
    arcpy.CheckOutExtension("Spatial")
           
    # Convert numpy.array to raster and save.
    # Peveden numpy.array na rastr a jeho uloen.
    if 1 in rb:
        depression_raster = arcpy.NumPyArrayToRaster(rb,arcpy.Point(LeftX,LeftY),vCell,value_to_nodata = 0)

        # Convert raster to integer format.
        # Peveden rastru na formt integer.
        outInt = Int(depression_raster)

        # Convert raster to polygon layer.
        # Peveden rastru na polygonovou vrstvu.
        arcpy.RasterToPolygon_conversion(outInt, output, "NO_SIMPLIFY")

        # Create a table and save it.
        # Vytvoen tabulky a jej uloen.
        arcpy.CreateTable_management(path+"\\", "Tab.dbf")
        path_tab = path+"\\"+"Tab.dbf"

        # Create new fields in the newly created table.
        # Vytvoen novch pol v nov vytvoen tabulce.
        arcpy.AddField_management( path_tab,"Num_dep","DOUBLE")
        arcpy.AddField_management( path_tab,"Max_Heigh","DOUBLE")
        arcpy.AddField_management( path_tab,"Min_Heigh","DOUBLE")
        arcpy.AddField_management( path_tab,"Depth","DOUBLE")
        arcpy.AddField_management( path_tab,"x","DOUBLE")
        arcpy.AddField_management( path_tab,"y","DOUBLE")

        # Adding information about each depression created table.
        # Doplnn daj o kad depresi v nov vytvoen tabulce.
        with arcpy.da.InsertCursor(path_tab, ["Num_dep","Max_Heigh","Min_Heigh","Depth","x","y"]) as cursor:
            for i in List1:
                cursor.insertRow([int(i[0]),float(i[1]),float(i[2]),float(i[3]),i[4],i[5]])

        # Linking the table to the polygon layer of depression.
        # Propojen tabulky s polygonovou vrstvou depres. 
        arcpy.JoinField_management(output, "GRIDCODE", path_tab, "Num_dep")

        # Deleting unnecessary fields in the table.
        # Smazan nepotebnch pol v tabulce.
        arcpy.DeleteField_management(output,["GRIDCODE","Field1"])

        # Deleting a created table.
        # Smazn vytvoen tabulky.
        os.remove(path_tab)
        os.remove(path_tab+".xml")
        os.remove(path_tab[:len(path_tab)-4]+".cpg")
    
    else:
        arcpy.AddMessage("-------------------------------------")
        arcpy.AddMessage("DEM does not contain any depression")
        arcpy.AddMessage("DEM neobsahuje dn uzaven deprese")

    if 1 in rc:
        flat_area_raster = arcpy.NumPyArrayToRaster(rc,arcpy.Point(LeftX,LeftY),vCell,value_to_nodata = 0)
    
        # Convert raster to integer format.
        # Peveden rastru na formt integer.
        outInt2 = Int(flat_area_raster)

        # Convert raster to polygon layer.
        # Peveden rastru na polygonovou vrstvu.
        arcpy.RasterToPolygon_conversion(outInt2, output_flat, "NO_SIMPLIFY")

        # Create a table and save it.
        # Vytvoen tabulky a jej uloen.
        arcpy.CreateTable_management(path+"\\", "Tab2.dbf")
        path_tab2 = path+"\\"+"Tab2.dbf"

        # Create new fields in the newly created table.
        # Vytvoen novch pol v nov vytvoen tabulce.
        arcpy.AddField_management( path_tab2,"Num_area","DOUBLE")
        arcpy.AddField_management( path_tab2,"Heigh","DOUBLE")

        # Adding information about each depression in created table.
        # Doplnn daj o kad depresi v nov vytvoen tabulce.
        with arcpy.da.InsertCursor(path_tab2, ["Num_area","Heigh"]) as cursor:
            for i in List2:
                cursor.insertRow([int(i[0]),float(i[1])])

        # Linking the table to the polygon layer of depression.
        # Propojen tabulky s polygonovou vrstvou depres.
        arcpy.JoinField_management(output_flat, "GRIDCODE", path_tab2, "Num_area")

        # Deleting unnecessary fields in the table.
        # Smazan nepotebnch pol v tabulce.
        arcpy.DeleteField_management(output_flat,["GRIDCODE","Field1"])
    
        # Deleting a created table.
        # Smazn vytvoen tabulky.
        os.remove(path_tab2)
        os.remove(path_tab2+".xml")
        os.remove(path_tab2[:len(path_tab2)-4]+".cpg")
        
    else:
        arcpy.AddMessage("-----------------------------------")
        arcpy.AddMessage("DEM does not contain any flat areas")
        arcpy.AddMessage("DEM neobsahuje dn ploiny")


pls = Dep_layer(raster, output, output_flat)






