# -*- coding: cp1250 -*-

"""
This script contains a "fill_DEM" function that loads the DEM raster, fills all of its closed
depressions to the elevation of their pour point. It is possible to enter a polygon layer
of depression and determine in advance what depression will be subject to filling. 

Tento skript obsahuje funkci "fill_DEM", kter nate rastr DEM a vypln vchny uzaven deprese
na hodnotu elevace jejich bodu peteen. Je mono zadat polygonovou vrstvu depres a pedem
urit jaky deprese budou pedmtem plnn.
"""

# Load modules.
# Naten modul.
import arcpy, numpy, sys, os, rta, heap, neighbors, border_of_raster

# Linking the parameters specified in  the window toolbox to the script.
# Propojen parametr zadanch v okn nstroje se skriptem.
raster = sys.argv[1]
output = sys.argv[2]
depression = sys.argv[3]

arcpy.env.overwriteOutput = True

def fill_DEM(raster , output, depression = False):

    arcpy.env.extent = raster 
    
    # Loads the raster to numpy.array.
    # Naten rastru do numpy.array
    aar_dem = rta.raster_to_numpy(raster)

    DEM = aar_dem[0]
    Xmax = aar_dem[1]
    Ymax = aar_dem[2]
    vCell = aar_dem[3]
    LeftX = aar_dem[4]
    LeftY = aar_dem[5]

    # Creates a zero array of the same size as the converted raster.
    # Vytvoen pole nul o stejn velikosti, jako peveden rastr.
    ra = numpy.zeros((Xmax,Ymax))   # Indicates which cells are processed. (Oznauje, kter buky jsou zpracovny.)

    # Creates a heap (x,y,z) sorted by the "z" value from values on the border of the  input raster.
    # Vytvoen haldy (x,y,z) azenou podle hodnoty "z" z hodnot na okraji vstupnho rastru.
    ha = []
    border_of_raster.border(DEM,ha,Xmax,Ymax)

    # Checks the neighbors of the first element in the heap.
    # Kontrola soused prvnho prvku v halde.
    while len(ha) != 0:
        ra[ha[0][0],ha[0][1]] = 1  # Sets the first element in the heap as processed. (Nastaven prvnho prvku v halde jako zpracovan.)
        sou = neighbors.neighbors_z(DEM,ha[0][0],ha[0][1],Xmax,Ymax)    # Detects coordinates and elevation of neighboring cells. (Zjitn souadnic a elevac sousednch bunk.)
        for i in sou:
            if ra[i[0],i[1]] != 1:  # Checks if the neighbor is not already processed. (Kontrola souseda jestli nen u zpracovan).
                if i[2] != -999:    # Check if the neighbors contains NoData. (Kontrola, jestli soused neobsahuje NoData.)
                    if i[2] < ha[0][2]:             
                        ra[i[0],i[1]] = 1
                        # Override neighbor elevation to processed cell elevation value.
                        # Pepsn elevace souseda na hodnotu elevace zpracovvan buky.
                        DEM[i[0],i[1]] = ha[0][2]
                    if i not in ha:     # If a neighbor is not in a heap, he adds it to the heap. (Pokud soused nen v hald, tak se pid do haldy.) 
                        heap.ins_el(ha, [i[0], i[1], DEM[i[0],i[1]]], index=2)
        # Removes the first element in the heap and replaces the next with the smallest elevation value.
        # Odstrann prvnho prvku v hald a nahrazen dalm s nejmen hodnotou.
        heap.ret_min(ha)


    if depression != "#":
        aar_dem_2 = rta.raster_to_numpy(raster)
        Org_DEM = aar_dem_2[0]

        # Converting the polygon layer to depression and then to numpy.array.
        # Peved polygonov vrstvy depres na rastr a nsledn na numpy.array.
        dep_raster = depression[:len(depression)-4]+".tif"
        arcpy.PolygonToRaster_conversion(depression, "Num_dep", dep_raster, "CELL_CENTER", cellsize = vCell)
        aar_dem_rb = rta.raster_to_numpy(dep_raster)
        Dep_raster = aar_dem_rb[0]

        # Creates a zero array of the same size as a array created from a input raster.
        # Vytvoen pole nul o stejn velikosti, jako pole vytvoen ze vstupnho rastru.
        Final_DEM = numpy.zeros((Xmax,Ymax))

        # Scan the raster layer for depression and assign the appropriate elevation value to the newly created raster.
        # Skenovn rastrov vrstvy depres a pidlen pslun hodnoty elevace nov vytvoenmu rastru.
        for i in range(Xmax):
            for j in range(Ymax):
                if Dep_raster[i,j] == -999:
                    Final_DEM[i,j] = Org_DEM[i,j]
                else:
                    Final_DEM[i,j] = DEM[i,j]


    # Convert numpy.array to raster and save.
    # Peveden numpy.array na rastr a jeho uloen.
    if depression != "#":
        fill_raster = arcpy.NumPyArrayToRaster(Final_DEM,arcpy.Point(LeftX,LeftY),vCell,value_to_nodata = -999) 
        fill_raster.save(output)
    else:
        fill_raster = arcpy.NumPyArrayToRaster(DEM,arcpy.Point(LeftX,LeftY),vCell,value_to_nodata = -999) 
        fill_raster.save(output)

    if depression != "#":
        # Removing the raster layer of depression.
        # Odstrann rastrov vrstvy depres
        os.remove(dep_raster[:len(dep_raster)-4] + ".tfw")
        os.remove(dep_raster + ".aux.xml")
        os.remove(dep_raster + ".ovr")
        os.remove(dep_raster + ".vat.cpg")
        os.remove(dep_raster + ".vat.dbf")
        os.remove(dep_raster + ".xml")
        os.remove(dep_raster)


pls = fill_DEM(raster, output, depression)   





